"""The pg commands module.

These commands are used in the following models:
MSO2

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - PG:AMPlitude {2.5|3.3|5V}
    - PG:AMPlitude?
    - PG:BIT:ONE {HIGH|LOW|TOGGLE|HIGH_Z}
    - PG:BIT:ONE?
    - PG:BIT:THREE {HIGH|LOW|TOGGLE|HIGH_Z}
    - PG:BIT:THREE?
    - PG:BIT:TWO {HIGH|LOW|TOGGLE|HIGH_Z}
    - PG:BIT:TWO?
    - PG:BIT:ZERO {HIGH|LOW|TOGGLE|HIGH_Z}
    - PG:BIT:ZERO?
    - PG:BITRate <NR3>
    - PG:BITRate?
    - PG:BURSt:CCOUnt <NR1>
    - PG:BURSt:CCOUnt?
    - PG:BURSt:TRIGger
    - PG:FILE:PATTern <QString>
    - PG:FILE:PATTern?
    - PG:OUTPut:MODe {CONTinuous|BURSt|OFF}
    - PG:OUTPut:MODe?
    - PG:PATTERNdefinition {MANual|FILE}
    - PG:PATTERNdefinition?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class PgPatterndefinition(SCPICmdWrite, SCPICmdRead):
    """The ``PG:PATTERNdefinition`` command.

    **Description:**
        - This command sets or queries the Pattern Generator definition. If set to manual mode, you
          can configure the bit independently. If set to file mode, you can set a data file that
          will be loaded into memory and generated as a sequence.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:PATTERNdefinition?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:PATTERNdefinition?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PG:PATTERNdefinition value`` command.

    **SCPI Syntax:**

    ::

        - PG:PATTERNdefinition {MANual|FILE}
        - PG:PATTERNdefinition?

    **Info:**
        - ``MANual`` sets the pattern definition source to Manual. This is the default value.
        - ``FILE`` sets the pattern definition source to File.
    """


class PgOutputMode(SCPICmdWrite, SCPICmdRead):
    """The ``PG:OUTPut:MODe`` command.

    **Description:**
        - This command sets or queries the Pattern Generator output mode.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:OUTPut:MODe?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:OUTPut:MODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PG:OUTPut:MODe value`` command.

    **SCPI Syntax:**

    ::

        - PG:OUTPut:MODe {CONTinuous|BURSt|OFF}
        - PG:OUTPut:MODe?

    **Info:**
        - ``CONTinuous`` sets the Pattern Generator output type to continuous.
        - ``BURSt`` sets the Pattern Generator output type to burst.
        - ``OFF`` sets the Pattern Generator output type to off. This is the default value.
    """


class PgOutput(SCPICmdRead):
    """The ``PG:OUTPut`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:OUTPut?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:OUTPut?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.mode``: The ``PG:OUTPut:MODe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._mode = PgOutputMode(device, f"{self._cmd_syntax}:MODe")

    @property
    def mode(self) -> PgOutputMode:
        """Return the ``PG:OUTPut:MODe`` command.

        **Description:**
            - This command sets or queries the Pattern Generator output mode.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:OUTPut:MODe?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:OUTPut:MODe?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``PG:OUTPut:MODe value`` command.

        **SCPI Syntax:**

        ::

            - PG:OUTPut:MODe {CONTinuous|BURSt|OFF}
            - PG:OUTPut:MODe?

        **Info:**
            - ``CONTinuous`` sets the Pattern Generator output type to continuous.
            - ``BURSt`` sets the Pattern Generator output type to burst.
            - ``OFF`` sets the Pattern Generator output type to off. This is the default value.
        """
        return self._mode


class PgFilePattern(SCPICmdWrite, SCPICmdRead):
    """The ``PG:FILE:PATTern`` command.

    **Description:**
        - This command sets or queries the path of your data file to generate a digital pattern in
          file mode. Only .csv files are supported.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:FILE:PATTern?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:FILE:PATTern?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PG:FILE:PATTern value`` command.

    **SCPI Syntax:**

    ::

        - PG:FILE:PATTern <QString>
        - PG:FILE:PATTern?

    **Info:**
        - ``<QString>`` is the file path that specifies the location of the specified instrument
          file.
    """

    _WRAP_ARG_WITH_QUOTES = True


class PgFile(SCPICmdRead):
    """The ``PG:FILE`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:FILE?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:FILE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.pattern``: The ``PG:FILE:PATTern`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._pattern = PgFilePattern(device, f"{self._cmd_syntax}:PATTern")

    @property
    def pattern(self) -> PgFilePattern:
        """Return the ``PG:FILE:PATTern`` command.

        **Description:**
            - This command sets or queries the path of your data file to generate a digital pattern
              in file mode. Only .csv files are supported.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:FILE:PATTern?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:FILE:PATTern?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``PG:FILE:PATTern value`` command.

        **SCPI Syntax:**

        ::

            - PG:FILE:PATTern <QString>
            - PG:FILE:PATTern?

        **Info:**
            - ``<QString>`` is the file path that specifies the location of the specified instrument
              file.
        """
        return self._pattern


class PgBurstTrigger(SCPICmdWriteNoArguments):
    """The ``PG:BURSt:TRIGger`` command.

    **Description:**
        - This command manually starts the burst pattern. This is only available when output is
          Burst.

    **Usage:**
        - Using the ``.write()`` method will send the ``PG:BURSt:TRIGger`` command.

    **SCPI Syntax:**

    ::

        - PG:BURSt:TRIGger
    """


class PgBurstCcount(SCPICmdWrite, SCPICmdRead):
    """The ``PG:BURSt:CCOUnt`` command.

    **Description:**
        - This command sets or queries the cycle count for Pattern Generator burst mode. This is
          only available when output is Burst.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:BURSt:CCOUnt?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:BURSt:CCOUnt?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PG:BURSt:CCOUnt value`` command.

    **SCPI Syntax:**

    ::

        - PG:BURSt:CCOUnt <NR1>
        - PG:BURSt:CCOUnt?

    **Info:**
        - ``<NR1>`` sets the cycle count for Pattern Generator burst mode. The minimum cycle value
          can be set is 1 and maximum cycles are 2000. The default value is 1.
    """


class PgBurst(SCPICmdRead):
    """The ``PG:BURSt`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:BURSt?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:BURSt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.ccount``: The ``PG:BURSt:CCOUnt`` command.
        - ``.trigger``: The ``PG:BURSt:TRIGger`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._ccount = PgBurstCcount(device, f"{self._cmd_syntax}:CCOUnt")
        self._trigger = PgBurstTrigger(device, f"{self._cmd_syntax}:TRIGger")

    @property
    def ccount(self) -> PgBurstCcount:
        """Return the ``PG:BURSt:CCOUnt`` command.

        **Description:**
            - This command sets or queries the cycle count for Pattern Generator burst mode. This is
              only available when output is Burst.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:BURSt:CCOUnt?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:BURSt:CCOUnt?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``PG:BURSt:CCOUnt value`` command.

        **SCPI Syntax:**

        ::

            - PG:BURSt:CCOUnt <NR1>
            - PG:BURSt:CCOUnt?

        **Info:**
            - ``<NR1>`` sets the cycle count for Pattern Generator burst mode. The minimum cycle
              value can be set is 1 and maximum cycles are 2000. The default value is 1.
        """
        return self._ccount

    @property
    def trigger(self) -> PgBurstTrigger:
        """Return the ``PG:BURSt:TRIGger`` command.

        **Description:**
            - This command manually starts the burst pattern. This is only available when output is
              Burst.

        **Usage:**
            - Using the ``.write()`` method will send the ``PG:BURSt:TRIGger`` command.

        **SCPI Syntax:**

        ::

            - PG:BURSt:TRIGger
        """
        return self._trigger


class PgBitrate(SCPICmdWrite, SCPICmdRead):
    """The ``PG:BITRate`` command.

    **Description:**
        - This command sets or queries the bit rate of data in the Pattern Generator. You can give
          any value, but it will take nearest possible value and generate the pattern.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:BITRate?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:BITRate?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PG:BITRate value`` command.

    **SCPI Syntax:**

    ::

        - PG:BITRate <NR3>
        - PG:BITRate?

    **Info:**
        - ``<NR3>`` sets the Pattern Generator bit rate. The minimum bit rate is 1 b/s and maximum
          bit rate is 25 Mb/s. The default value is 125000 b/s.
    """


class PgBitZero(SCPICmdWrite, SCPICmdRead):
    """The ``PG:BIT:ZERO`` command.

    **Description:**
        - This command sets or queries the output value of Pattern Generator bit 0.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:BIT:ZERO?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:BIT:ZERO?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PG:BIT:ZERO value`` command.

    **SCPI Syntax:**

    ::

        - PG:BIT:ZERO {HIGH|LOW|TOGGLE|HIGH_Z}
        - PG:BIT:ZERO?

    **Info:**
        - ``LOW`` sets the Pattern Generator bit value to Low.
        - ``HIGH`` sets the Pattern Generator bit value to High.
        - ``TOGGLE`` sets the Pattern Generator bit value to Toggle.
        - ``HIGH_Z`` sets the Pattern Generator bit value to Hi-Z. Only available for Continuos
          mode.
    """


class PgBitTwo(SCPICmdWrite, SCPICmdRead):
    """The ``PG:BIT:TWO`` command.

    **Description:**
        - This command sets or queries the output value of Pattern Generator bit 2.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:BIT:TWO?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:BIT:TWO?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PG:BIT:TWO value`` command.

    **SCPI Syntax:**

    ::

        - PG:BIT:TWO {HIGH|LOW|TOGGLE|HIGH_Z}
        - PG:BIT:TWO?

    **Info:**
        - ``LOW`` sets the Pattern Generator bit value to Low.
        - ``HIGH`` sets the Pattern Generator bit value to High.
        - ``TOGGLE`` sets the Pattern Generator bit value to Toggle.
        - ``HIGH_Z`` sets the Pattern Generator bit value to Hi-Z. Only available for Continuos
          mode.
    """


class PgBitThree(SCPICmdWrite, SCPICmdRead):
    """The ``PG:BIT:THREE`` command.

    **Description:**
        - This command sets or queries the output value of Pattern Generator bit 3.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:BIT:THREE?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:BIT:THREE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PG:BIT:THREE value`` command.

    **SCPI Syntax:**

    ::

        - PG:BIT:THREE {HIGH|LOW|TOGGLE|HIGH_Z}
        - PG:BIT:THREE?

    **Info:**
        - ``LOW`` sets the Pattern Generator bit value to Low.
        - ``HIGH`` sets the Pattern Generator bit value to High.
        - ``TOGGLE`` sets the Pattern Generator bit value to Toggle.
        - ``HIGH_Z`` sets the Pattern Generator bit value to Hi-Z. Only available for Continuos
          mode.
    """


class PgBitOne(SCPICmdWrite, SCPICmdRead):
    """The ``PG:BIT:ONE`` command.

    **Description:**
        - This command sets or queries the output value of Pattern Generator bit 1.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:BIT:ONE?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:BIT:ONE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PG:BIT:ONE value`` command.

    **SCPI Syntax:**

    ::

        - PG:BIT:ONE {HIGH|LOW|TOGGLE|HIGH_Z}
        - PG:BIT:ONE?

    **Info:**
        - ``LOW`` sets the Pattern Generator bit value to Low.
        - ``HIGH`` sets the Pattern Generator bit value to High.
        - ``TOGGLE`` sets the Pattern Generator bit value to Toggle.
        - ``HIGH_Z`` sets the Pattern Generator bit value to Hi-Z. Only available for Continuos
          mode.
    """


class PgBit(SCPICmdRead):
    """The ``PG:BIT`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:BIT?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:BIT?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.one``: The ``PG:BIT:ONE`` command.
        - ``.three``: The ``PG:BIT:THREE`` command.
        - ``.two``: The ``PG:BIT:TWO`` command.
        - ``.zero``: The ``PG:BIT:ZERO`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._one = PgBitOne(device, f"{self._cmd_syntax}:ONE")
        self._three = PgBitThree(device, f"{self._cmd_syntax}:THREE")
        self._two = PgBitTwo(device, f"{self._cmd_syntax}:TWO")
        self._zero = PgBitZero(device, f"{self._cmd_syntax}:ZERO")

    @property
    def one(self) -> PgBitOne:
        """Return the ``PG:BIT:ONE`` command.

        **Description:**
            - This command sets or queries the output value of Pattern Generator bit 1.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:BIT:ONE?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:BIT:ONE?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``PG:BIT:ONE value`` command.

        **SCPI Syntax:**

        ::

            - PG:BIT:ONE {HIGH|LOW|TOGGLE|HIGH_Z}
            - PG:BIT:ONE?

        **Info:**
            - ``LOW`` sets the Pattern Generator bit value to Low.
            - ``HIGH`` sets the Pattern Generator bit value to High.
            - ``TOGGLE`` sets the Pattern Generator bit value to Toggle.
            - ``HIGH_Z`` sets the Pattern Generator bit value to Hi-Z. Only available for Continuos
              mode.
        """
        return self._one

    @property
    def three(self) -> PgBitThree:
        """Return the ``PG:BIT:THREE`` command.

        **Description:**
            - This command sets or queries the output value of Pattern Generator bit 3.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:BIT:THREE?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:BIT:THREE?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``PG:BIT:THREE value`` command.

        **SCPI Syntax:**

        ::

            - PG:BIT:THREE {HIGH|LOW|TOGGLE|HIGH_Z}
            - PG:BIT:THREE?

        **Info:**
            - ``LOW`` sets the Pattern Generator bit value to Low.
            - ``HIGH`` sets the Pattern Generator bit value to High.
            - ``TOGGLE`` sets the Pattern Generator bit value to Toggle.
            - ``HIGH_Z`` sets the Pattern Generator bit value to Hi-Z. Only available for Continuos
              mode.
        """
        return self._three

    @property
    def two(self) -> PgBitTwo:
        """Return the ``PG:BIT:TWO`` command.

        **Description:**
            - This command sets or queries the output value of Pattern Generator bit 2.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:BIT:TWO?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:BIT:TWO?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``PG:BIT:TWO value`` command.

        **SCPI Syntax:**

        ::

            - PG:BIT:TWO {HIGH|LOW|TOGGLE|HIGH_Z}
            - PG:BIT:TWO?

        **Info:**
            - ``LOW`` sets the Pattern Generator bit value to Low.
            - ``HIGH`` sets the Pattern Generator bit value to High.
            - ``TOGGLE`` sets the Pattern Generator bit value to Toggle.
            - ``HIGH_Z`` sets the Pattern Generator bit value to Hi-Z. Only available for Continuos
              mode.
        """
        return self._two

    @property
    def zero(self) -> PgBitZero:
        """Return the ``PG:BIT:ZERO`` command.

        **Description:**
            - This command sets or queries the output value of Pattern Generator bit 0.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:BIT:ZERO?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:BIT:ZERO?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``PG:BIT:ZERO value`` command.

        **SCPI Syntax:**

        ::

            - PG:BIT:ZERO {HIGH|LOW|TOGGLE|HIGH_Z}
            - PG:BIT:ZERO?

        **Info:**
            - ``LOW`` sets the Pattern Generator bit value to Low.
            - ``HIGH`` sets the Pattern Generator bit value to High.
            - ``TOGGLE`` sets the Pattern Generator bit value to Toggle.
            - ``HIGH_Z`` sets the Pattern Generator bit value to Hi-Z. Only available for Continuos
              mode.
        """
        return self._zero


class PgAmplitude(SCPICmdWrite, SCPICmdRead):
    """The ``PG:AMPlitude`` command.

    **Description:**
        - This command sets or queries the Pattern Generator output voltage. You can give any value,
          but the command will replace it with the nearest valid value (2.5, 3.3 and 5). In burst
          mode, only 5 V is supported.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG:AMPlitude?`` query.
        - Using the ``.verify(value)`` method will send the ``PG:AMPlitude?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PG:AMPlitude value`` command.

    **SCPI Syntax:**

    ::

        - PG:AMPlitude {2.5|3.3|5V}
        - PG:AMPlitude?

    **Info:**
        - ``2.5`` sets the Pattern Generator amplitude to 2.5 V.
        - ``3.3`` sets the Pattern Generator amplitude to 3.3 V.
        - ``5`` sets the Pattern Generator amplitude to 5 V. This is the default value.
    """


class Pg(SCPICmdRead):
    """The ``PG`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``PG?`` query.
        - Using the ``.verify(value)`` method will send the ``PG?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.amplitude``: The ``PG:AMPlitude`` command.
        - ``.bit``: The ``PG:BIT`` command tree.
        - ``.bitrate``: The ``PG:BITRate`` command.
        - ``.burst``: The ``PG:BURSt`` command tree.
        - ``.file``: The ``PG:FILE`` command tree.
        - ``.output``: The ``PG:OUTPut`` command tree.
        - ``.patterndefinition``: The ``PG:PATTERNdefinition`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "PG") -> None:
        super().__init__(device, cmd_syntax)
        self._amplitude = PgAmplitude(device, f"{self._cmd_syntax}:AMPlitude")
        self._bit = PgBit(device, f"{self._cmd_syntax}:BIT")
        self._bitrate = PgBitrate(device, f"{self._cmd_syntax}:BITRate")
        self._burst = PgBurst(device, f"{self._cmd_syntax}:BURSt")
        self._file = PgFile(device, f"{self._cmd_syntax}:FILE")
        self._output = PgOutput(device, f"{self._cmd_syntax}:OUTPut")
        self._patterndefinition = PgPatterndefinition(
            device, f"{self._cmd_syntax}:PATTERNdefinition"
        )

    @property
    def amplitude(self) -> PgAmplitude:
        """Return the ``PG:AMPlitude`` command.

        **Description:**
            - This command sets or queries the Pattern Generator output voltage. You can give any
              value, but the command will replace it with the nearest valid value (2.5, 3.3 and 5).
              In burst mode, only 5 V is supported.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:AMPlitude?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:AMPlitude?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``PG:AMPlitude value`` command.

        **SCPI Syntax:**

        ::

            - PG:AMPlitude {2.5|3.3|5V}
            - PG:AMPlitude?

        **Info:**
            - ``2.5`` sets the Pattern Generator amplitude to 2.5 V.
            - ``3.3`` sets the Pattern Generator amplitude to 3.3 V.
            - ``5`` sets the Pattern Generator amplitude to 5 V. This is the default value.
        """
        return self._amplitude

    @property
    def bit(self) -> PgBit:
        """Return the ``PG:BIT`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:BIT?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:BIT?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.one``: The ``PG:BIT:ONE`` command.
            - ``.three``: The ``PG:BIT:THREE`` command.
            - ``.two``: The ``PG:BIT:TWO`` command.
            - ``.zero``: The ``PG:BIT:ZERO`` command.
        """
        return self._bit

    @property
    def bitrate(self) -> PgBitrate:
        """Return the ``PG:BITRate`` command.

        **Description:**
            - This command sets or queries the bit rate of data in the Pattern Generator. You can
              give any value, but it will take nearest possible value and generate the pattern.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:BITRate?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:BITRate?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``PG:BITRate value`` command.

        **SCPI Syntax:**

        ::

            - PG:BITRate <NR3>
            - PG:BITRate?

        **Info:**
            - ``<NR3>`` sets the Pattern Generator bit rate. The minimum bit rate is 1 b/s and
              maximum bit rate is 25 Mb/s. The default value is 125000 b/s.
        """
        return self._bitrate

    @property
    def burst(self) -> PgBurst:
        """Return the ``PG:BURSt`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:BURSt?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:BURSt?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.ccount``: The ``PG:BURSt:CCOUnt`` command.
            - ``.trigger``: The ``PG:BURSt:TRIGger`` command.
        """
        return self._burst

    @property
    def file(self) -> PgFile:
        """Return the ``PG:FILE`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:FILE?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:FILE?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.pattern``: The ``PG:FILE:PATTern`` command.
        """
        return self._file

    @property
    def output(self) -> PgOutput:
        """Return the ``PG:OUTPut`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:OUTPut?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:OUTPut?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.mode``: The ``PG:OUTPut:MODe`` command.
        """
        return self._output

    @property
    def patterndefinition(self) -> PgPatterndefinition:
        """Return the ``PG:PATTERNdefinition`` command.

        **Description:**
            - This command sets or queries the Pattern Generator definition. If set to manual mode,
              you can configure the bit independently. If set to file mode, you can set a data file
              that will be loaded into memory and generated as a sequence.

        **Usage:**
            - Using the ``.query()`` method will send the ``PG:PATTERNdefinition?`` query.
            - Using the ``.verify(value)`` method will send the ``PG:PATTERNdefinition?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``PG:PATTERNdefinition value``
              command.

        **SCPI Syntax:**

        ::

            - PG:PATTERNdefinition {MANual|FILE}
            - PG:PATTERNdefinition?

        **Info:**
            - ``MANual`` sets the pattern definition source to Manual. This is the default value.
            - ``FILE`` sets the pattern definition source to File.
        """
        return self._patterndefinition



"""Systems Switches package init file."""
from tm_devices.drivers.pi.systems_switches.ss3706a import SS3706A
from tm_devices.drivers.pi.systems_switches.systems_switch import SystemsSwitch

__all__ = [
    "SystemsSwitch",
    "SS3706A",
]



"""A mixin class providing common methods and attributes for devices with installable licenses."""

from abc import ABC, abstractmethod
from functools import cached_property
from typing import final, Tuple


class LicensedMixin(ABC):
    """A mixin class which adds methods and properties for handling licenses."""

    @cached_property
    @abstractmethod
    def license_list(self) -> Tuple[str, ...]:
        """Return the list of licenses installed on the device."""

    @final
    def has_license(self, license_name: str) -> bool:
        """Check if the given license name is in the license list of the device.

        Args:
            license_name: The name of the license to check for in the license list.

        Returns:
            A boolean indicating if the license name is in the list.
        """
        return license_name in self.license_list



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The dataqueue commands module.

These commands are used in the following models:
DAQ6510, DMM6500, DMM7510, SMU2450, SMU2460, SMU2461, SMU2470, SMU2601B, SMU2602B, SMU2604B,
SMU2606B, SMU2611B, SMU2612B, SMU2614B, SMU2634B, SMU2635B, SMU2636B, SMU2651A, SMU2657A, SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - dataqueue.add()
    - dataqueue.clear()
    - dataqueue.count
    - dataqueue.next()
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class Dataqueue(BaseTSPCmd):
    """The ``dataqueue`` command tree.

    Constants:
        - ``.CAPACITY``: The maximum number of entries that you can store in the data queue.

    Properties/methods:
        - ``.add()``: The ``dataqueue.add()`` function.
        - ``.clear()``: The ``dataqueue.clear()`` function.
        - ``.count``: The ``dataqueue.count`` attribute.
        - ``.next()``: The ``dataqueue.next()`` function.
    """

    CAPACITY = "dataqueue.CAPACITY"
    """str: The maximum number of entries that you can store in the data queue."""

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "dataqueue") -> None:
        super().__init__(device, cmd_syntax)

    @property
    def count(self) -> str:
        """Access the ``dataqueue.count`` attribute.

        **Description:**
            - This attribute contains the number of items in the data queue.

        **Usage:**
            - Accessing this property will send the ``print(dataqueue.count)`` query.

        **TSP Syntax:**

        ::

            - print(dataqueue.count)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".count"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.count)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.count`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def add(self, value: str, timeout: Optional[float] = None) -> str:
        """Run the ``dataqueue.add()`` function.

        **Description:**
            - This function adds an entry to the data queue.

        **TSP Syntax:**

        ::

            - dataqueue.add()

        Args:
            value: The data item to add; value can be of any type.
            timeout (optional): The maximum number of seconds to wait for space in the data queue.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    value,
                    timeout,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.add({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.add()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def clear(self) -> None:
        """Run the ``dataqueue.clear()`` function.

        **Description:**
            - This function clears the data queue.

        **TSP Syntax:**

        ::

            - dataqueue.clear()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def next(self, timeout: Optional[float] = None) -> str:
        """Run the ``dataqueue.next()`` function.

        **Description:**
            - This function removes the next entry from the data queue.

        **TSP Syntax:**

        ::

            - dataqueue.next()

        Args:
            timeout (optional): The number of seconds to wait for data in the queue.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (timeout,) if x is not None)
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.next({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.next()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The recall commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - RECAll:MASK <QString>
    - RECAll:SETUp {FACtory|<NR1>|<file path>}
    - RECAll:SETUp:DESKew {LOCK|UNLOCK}
    - RECAll:SETUp:DESKew?
    - RECAll:WAVEform <file path>,REF<x>
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class RecallWaveform(SCPICmdWrite):
    r"""The ``RECAll:WAVEform`` command.

    **Description:**
        - This command (no query form) recalls a stored waveform to a reference location. This
          command is equivalent to selecting Recall from the File menu, and then pressing the
          Waveform button.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``RECAll:WAVEform value`` command.

    **SCPI Syntax:**

    ::

        - RECAll:WAVEform <file path>,REF<x>

    **Info:**
        - ``<file path>`` specifies a location for a stored waveform file. <file path> is a quoted
          string that defines the file name and path. Input the file path using the form
          ``<drive>:<dir>``/<filename>.<drive> and one or more <dir>s are optional. If you do not
          specify them, the instrument will read the waveform from the default directory
          (C:UsersPublicTektronixTekScope\|Waveforms). <filename> stands for a file name of up to
          128 characters (use of wildcard characters in filenames is not supported). File name
          extensions are not required but are highly recommended.
        - ``REF<x>`` specifies a location in internal reference memory. Reference memory location
          values range from 1 through 4.
    """


class RecallSetupDeskew(SCPICmdWrite, SCPICmdRead):
    """The ``RECAll:SETUp:DESKew`` command.

    **Description:**
        - This command sets or queries the deskew values that are affected by a default setup or a
          recalled setup.

    **Usage:**
        - Using the ``.query()`` method will send the ``RECAll:SETUp:DESKew?`` query.
        - Using the ``.verify(value)`` method will send the ``RECAll:SETUp:DESKew?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``RECAll:SETUp:DESKew value`` command.

    **SCPI Syntax:**

    ::

        - RECAll:SETUp:DESKew {LOCK|UNLOCK}
        - RECAll:SETUp:DESKew?

    **Info:**
        - ``LOCK`` a default or a recall setup will not change the deskew settings when this option
          is selected.
        - ``UNLOCK`` a default setup will reset the deskew settings to factory values and a recall
          setup will apply the setup deskew values to the recalled oscilloscope state.
    """


class RecallSetup(SCPICmdWrite, SCPICmdRead):
    """The ``RECAll:SETUp`` command.

    **Description:**
        - Restores the state of the oscilloscope from a copy of the settings stored in memory. The
          settings are stored using the ``*SAV`` command.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``RECAll:SETUp value`` command.

    **SCPI Syntax:**

    ::

        - RECAll:SETUp {FACtory|<NR1>|<file path>}

    **Info:**
        - ``FACtory`` restores the factory setup.
        - ``<NR1>`` is a value in the range from 1 to 10, which specifies a saved setup storage
          location.
        - ``<file path>`` specifies a location for an oscilloscope setup file. <file path> is a
          quoted string that defines the file name and path. Input the file path using the form
          <drive>:/<dir>/<filename>.<extension> and one or <dir>s are optional. If you do not
          specify them, the oscilloscope will read the file from the default directory (see
          ``FILESYSTEM:CWD``). <filename> stands for a filename; the use of wildcard characters in
          filenames is not supported. Filename extensions are not required, but highly recommended.

    Properties:
        - ``.deskew``: The ``RECAll:SETUp:DESKew`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._deskew = RecallSetupDeskew(device, f"{self._cmd_syntax}:DESKew")

    @property
    def deskew(self) -> RecallSetupDeskew:
        """Return the ``RECAll:SETUp:DESKew`` command.

        **Description:**
            - This command sets or queries the deskew values that are affected by a default setup or
              a recalled setup.

        **Usage:**
            - Using the ``.query()`` method will send the ``RECAll:SETUp:DESKew?`` query.
            - Using the ``.verify(value)`` method will send the ``RECAll:SETUp:DESKew?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``RECAll:SETUp:DESKew value``
              command.

        **SCPI Syntax:**

        ::

            - RECAll:SETUp:DESKew {LOCK|UNLOCK}
            - RECAll:SETUp:DESKew?

        **Info:**
            - ``LOCK`` a default or a recall setup will not change the deskew settings when this
              option is selected.
            - ``UNLOCK`` a default setup will reset the deskew settings to factory values and a
              recall setup will apply the setup deskew values to the recalled oscilloscope state.
        """
        return self._deskew


class RecallMask(SCPICmdWrite):
    """The ``RECAll:MASK`` command.

    **Description:**
        - This command recalls the mask from a specified file that was saved to the current working
          directory using the ``SAVE:MASK`` command. A series of examples showing how to use mask
          commands for typical tasks is included in an appendix.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``RECAll:MASK value`` command.

    **SCPI Syntax:**

    ::

        - RECAll:MASK <QString>

    **Info:**
        - ``QString`` is a quoted string that is the name of the mask definition being recalled from
          the current working directory.
    """

    _WRAP_ARG_WITH_QUOTES = True


class Recall(SCPICmdRead):
    """The ``RECAll`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``RECAll?`` query.
        - Using the ``.verify(value)`` method will send the ``RECAll?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.mask``: The ``RECAll:MASK`` command.
        - ``.setup``: The ``RECAll:SETUp`` command.
        - ``.waveform``: The ``RECAll:WAVEform`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "RECAll") -> None:
        super().__init__(device, cmd_syntax)
        self._mask = RecallMask(device, f"{self._cmd_syntax}:MASK")
        self._setup = RecallSetup(device, f"{self._cmd_syntax}:SETUp")
        self._waveform = RecallWaveform(device, f"{self._cmd_syntax}:WAVEform")

    @property
    def mask(self) -> RecallMask:
        """Return the ``RECAll:MASK`` command.

        **Description:**
            - This command recalls the mask from a specified file that was saved to the current
              working directory using the ``SAVE:MASK`` command. A series of examples showing how to
              use mask commands for typical tasks is included in an appendix.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``RECAll:MASK value`` command.

        **SCPI Syntax:**

        ::

            - RECAll:MASK <QString>

        **Info:**
            - ``QString`` is a quoted string that is the name of the mask definition being recalled
              from the current working directory.
        """
        return self._mask

    @property
    def setup(self) -> RecallSetup:
        """Return the ``RECAll:SETUp`` command.

        **Description:**
            - Restores the state of the oscilloscope from a copy of the settings stored in memory.
              The settings are stored using the ``*SAV`` command.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``RECAll:SETUp value`` command.

        **SCPI Syntax:**

        ::

            - RECAll:SETUp {FACtory|<NR1>|<file path>}

        **Info:**
            - ``FACtory`` restores the factory setup.
            - ``<NR1>`` is a value in the range from 1 to 10, which specifies a saved setup storage
              location.
            - ``<file path>`` specifies a location for an oscilloscope setup file. <file path> is a
              quoted string that defines the file name and path. Input the file path using the form
              <drive>:/<dir>/<filename>.<extension> and one or <dir>s are optional. If you do not
              specify them, the oscilloscope will read the file from the default directory (see
              ``FILESYSTEM:CWD``). <filename> stands for a filename; the use of wildcard characters
              in filenames is not supported. Filename extensions are not required, but highly
              recommended.

        Sub-properties:
            - ``.deskew``: The ``RECAll:SETUp:DESKew`` command.
        """
        return self._setup

    @property
    def waveform(self) -> RecallWaveform:
        r"""Return the ``RECAll:WAVEform`` command.

        **Description:**
            - This command (no query form) recalls a stored waveform to a reference location. This
              command is equivalent to selecting Recall from the File menu, and then pressing the
              Waveform button.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``RECAll:WAVEform value`` command.

        **SCPI Syntax:**

        ::

            - RECAll:WAVEform <file path>,REF<x>

        **Info:**
            - ``<file path>`` specifies a location for a stored waveform file. <file path> is a
              quoted string that defines the file name and path. Input the file path using the form
              ``<drive>:<dir>``/<filename>.<drive> and one or more <dir>s are optional. If you do
              not specify them, the instrument will read the waveform from the default directory
              (C:UsersPublicTektronixTekScope\|Waveforms). <filename> stands for a file name of up
              to 128 characters (use of wildcard characters in filenames is not supported). File
              name extensions are not required but are highly recommended.
            - ``REF<x>`` specifies a location in internal reference memory. Reference memory
              location values range from 1 through 4.
        """
        return self._waveform



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The errorqueue commands module.

These commands are used in the following models:
SMU2601B_Pulse

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - errorqueue.next()
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class Errorqueue(BaseTSPCmd):
    """The ``errorqueue`` command tree.

    Properties/methods:
        - ``.next()``: The ``errorqueue.next()`` function.
    """

    def __init__(
        self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "errorqueue"
    ) -> None:
        super().__init__(device, cmd_syntax)

    def next(self) -> str:
        """Run the ``errorqueue.next()`` function.

        **Description:**
            - This function reads the oldest entry from the error queue and removes it from the
              queue.

        **TSP Syntax:**

        ::

            - errorqueue.next()

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.next())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.next()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The bell commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - BELl
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Bell(SCPICmdWriteNoArguments):
    """The ``BELl`` command.

    **Description:**
        - This command was previously used to beep an audio indicator and is provided for backward
          compatibility.

    **Usage:**
        - Using the ``.write()`` method will send the ``BELl`` command.

    **SCPI Syntax:**

    ::

        - BELl
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "BELl") -> None:
        super().__init__(device, cmd_syntax)



# pylint: disable=too-many-lines
# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The scan commands module.

These commands are used in the following models:
SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - scan.abort()
    - scan.add()
    - scan.addimagestep()
    - scan.addwrite()
    - scan.background()
    - scan.bypass
    - scan.create()
    - scan.execute()
    - scan.list()
    - scan.measurecount
    - scan.mode
    - scan.nobufferbackground()
    - scan.nobufferexecute()
    - scan.reset()
    - scan.scancount
    - scan.state()
    - scan.stepcount
    - scan.trigger.arm.clear()
    - scan.trigger.arm.set()
    - scan.trigger.arm.stimulus
    - scan.trigger.channel.clear()
    - scan.trigger.channel.set()
    - scan.trigger.channel.stimulus
    - scan.trigger.clear()
    - scan.trigger.measure.clear()
    - scan.trigger.measure.set()
    - scan.trigger.measure.stimulus
    - scan.trigger.sequence.clear()
    - scan.trigger.sequence.set()
    - scan.trigger.sequence.stimulus
"""
from typing import Optional, TYPE_CHECKING, Union

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class ScanTriggerSequence(BaseTSPCmd):
    """The ``scan.trigger.sequence`` command tree.

    Properties/methods:
        - ``.clear()``: The ``scan.trigger.sequence.clear()`` function.
        - ``.set()``: The ``scan.trigger.sequence.set()`` function.
        - ``.stimulus``: The ``scan.trigger.sequence.stimulus`` attribute.
    """

    @property
    def stimulus(self) -> str:
        """Access the ``scan.trigger.sequence.stimulus`` attribute.

        **Description:**
            - This attribute selects the trigger stimulus for the sequence event detector.

        **Usage:**
            - Accessing this property will send the ``print(scan.trigger.sequence.stimulus)`` query.
            - Setting this property to a value will send the
              ``scan.trigger.sequence.stimulus = value`` command.

        **TSP Syntax:**

        ::

            - scan.trigger.sequence.stimulus = value
            - print(scan.trigger.sequence.stimulus)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stimulus"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stimulus)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @stimulus.setter
    def stimulus(self, value: Union[str, float]) -> None:
        """Access the ``scan.trigger.sequence.stimulus`` attribute.

        **Description:**
            - This attribute selects the trigger stimulus for the sequence event detector.

        **Usage:**
            - Accessing this property will send the ``print(scan.trigger.sequence.stimulus)`` query.
            - Setting this property to a value will send the
              ``scan.trigger.sequence.stimulus = value`` command.

        **TSP Syntax:**

        ::

            - scan.trigger.sequence.stimulus = value
            - print(scan.trigger.sequence.stimulus)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".stimulus", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.stimulus = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def clear(self) -> None:
        """Run the ``scan.trigger.sequence.clear()`` function.

        **Description:**
            - This function clears the sequence event detector.

        **TSP Syntax:**

        ::

            - scan.trigger.sequence.clear()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def set_(self) -> None:
        """Run the ``scan.trigger.sequence.set()`` function.

        **Description:**
            - This function sets the sequence even detector to the detected state.

        **TSP Syntax:**

        ::

            - scan.trigger.sequence.set()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.set()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.set()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class ScanTriggerMeasure(BaseTSPCmd):
    """The ``scan.trigger.measure`` command tree.

    Properties/methods:
        - ``.clear()``: The ``scan.trigger.measure.clear()`` function.
        - ``.set()``: The ``scan.trigger.measure.set()`` function.
        - ``.stimulus``: The ``scan.trigger.measure.stimulus`` attribute.
    """

    @property
    def stimulus(self) -> str:
        """Access the ``scan.trigger.measure.stimulus`` attribute.

        **Description:**
            - This attribute selects the trigger stimulus of the event detector trigger.

        **Usage:**
            - Accessing this property will send the ``print(scan.trigger.measure.stimulus)`` query.
            - Setting this property to a value will send the
              ``scan.trigger.measure.stimulus = value`` command.

        **TSP Syntax:**

        ::

            - scan.trigger.measure.stimulus = value
            - print(scan.trigger.measure.stimulus)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stimulus"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stimulus)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @stimulus.setter
    def stimulus(self, value: Union[str, float]) -> None:
        """Access the ``scan.trigger.measure.stimulus`` attribute.

        **Description:**
            - This attribute selects the trigger stimulus of the event detector trigger.

        **Usage:**
            - Accessing this property will send the ``print(scan.trigger.measure.stimulus)`` query.
            - Setting this property to a value will send the
              ``scan.trigger.measure.stimulus = value`` command.

        **TSP Syntax:**

        ::

            - scan.trigger.measure.stimulus = value
            - print(scan.trigger.measure.stimulus)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".stimulus", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.stimulus = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def clear(self) -> None:
        """Run the ``scan.trigger.measure.clear()`` function.

        **Description:**
            - This function clears the measure event detector.

        **TSP Syntax:**

        ::

            - scan.trigger.measure.clear()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def set_(self) -> None:
        """Run the ``scan.trigger.measure.set()`` function.

        **Description:**
            - This function sets the measurement event detector to the detected state.

        **TSP Syntax:**

        ::

            - scan.trigger.measure.set()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.set()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.set()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class ScanTriggerChannel(BaseTSPCmd):
    """The ``scan.trigger.channel`` command tree.

    Properties/methods:
        - ``.clear()``: The ``scan.trigger.channel.clear()`` function.
        - ``.set()``: The ``scan.trigger.channel.set()`` function.
        - ``.stimulus``: The ``scan.trigger.channel.stimulus`` attribute.
    """

    @property
    def stimulus(self) -> str:
        """Access the ``scan.trigger.channel.stimulus`` attribute.

        **Description:**
            - This attribute determines which trigger events cause the channel actions to occur.

        **Usage:**
            - Accessing this property will send the ``print(scan.trigger.channel.stimulus)`` query.
            - Setting this property to a value will send the
              ``scan.trigger.channel.stimulus = value`` command.

        **TSP Syntax:**

        ::

            - scan.trigger.channel.stimulus = value
            - print(scan.trigger.channel.stimulus)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stimulus"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stimulus)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @stimulus.setter
    def stimulus(self, value: Union[str, float]) -> None:
        """Access the ``scan.trigger.channel.stimulus`` attribute.

        **Description:**
            - This attribute determines which trigger events cause the channel actions to occur.

        **Usage:**
            - Accessing this property will send the ``print(scan.trigger.channel.stimulus)`` query.
            - Setting this property to a value will send the
              ``scan.trigger.channel.stimulus = value`` command.

        **TSP Syntax:**

        ::

            - scan.trigger.channel.stimulus = value
            - print(scan.trigger.channel.stimulus)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".stimulus", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.stimulus = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def clear(self) -> None:
        """Run the ``scan.trigger.channel.clear()`` function.

        **Description:**
            - This function clears the channel event detector.

        **TSP Syntax:**

        ::

            - scan.trigger.channel.clear()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def set_(self) -> None:
        """Run the ``scan.trigger.channel.set()`` function.

        **Description:**
            - This function sets the channel event detector to the detected state.

        **TSP Syntax:**

        ::

            - scan.trigger.channel.set()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.set()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.set()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class ScanTriggerArm(BaseTSPCmd):
    """The ``scan.trigger.arm`` command tree.

    Properties/methods:
        - ``.clear()``: The ``scan.trigger.arm.clear()`` function.
        - ``.set()``: The ``scan.trigger.arm.set()`` function.
        - ``.stimulus``: The ``scan.trigger.arm.stimulus`` attribute.
    """

    @property
    def stimulus(self) -> str:
        """Access the ``scan.trigger.arm.stimulus`` attribute.

        **Description:**
            - This attribute determines which event starts the scan.

        **Usage:**
            - Accessing this property will send the ``print(scan.trigger.arm.stimulus)`` query.
            - Setting this property to a value will send the ``scan.trigger.arm.stimulus = value``
              command.

        **TSP Syntax:**

        ::

            - scan.trigger.arm.stimulus = value
            - print(scan.trigger.arm.stimulus)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stimulus"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stimulus)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @stimulus.setter
    def stimulus(self, value: Union[str, float]) -> None:
        """Access the ``scan.trigger.arm.stimulus`` attribute.

        **Description:**
            - This attribute determines which event starts the scan.

        **Usage:**
            - Accessing this property will send the ``print(scan.trigger.arm.stimulus)`` query.
            - Setting this property to a value will send the ``scan.trigger.arm.stimulus = value``
              command.

        **TSP Syntax:**

        ::

            - scan.trigger.arm.stimulus = value
            - print(scan.trigger.arm.stimulus)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".stimulus", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.stimulus = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def clear(self) -> None:
        """Run the ``scan.trigger.arm.clear()`` function.

        **Description:**
            - This function clears the arm event detector.

        **TSP Syntax:**

        ::

            - scan.trigger.arm.clear()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def set_(self) -> None:
        """Run the ``scan.trigger.arm.set()`` function.

        **Description:**
            - This function sets the arm event detector to the detected state.

        **TSP Syntax:**

        ::

            - scan.trigger.arm.set()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.set()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.set()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class ScanTrigger(BaseTSPCmd):
    """The ``scan.trigger`` command tree.

    Properties/methods:
        - ``.arm``: The ``scan.trigger.arm`` command tree.
        - ``.channel``: The ``scan.trigger.channel`` command tree.
        - ``.clear()``: The ``scan.trigger.clear()`` function.
        - ``.measure``: The ``scan.trigger.measure`` command tree.
        - ``.sequence``: The ``scan.trigger.sequence`` command tree.
    """

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._arm = ScanTriggerArm(device, f"{self._cmd_syntax}.arm")
        self._channel = ScanTriggerChannel(device, f"{self._cmd_syntax}.channel")
        self._measure = ScanTriggerMeasure(device, f"{self._cmd_syntax}.measure")
        self._sequence = ScanTriggerSequence(device, f"{self._cmd_syntax}.sequence")

    @property
    def arm(self) -> ScanTriggerArm:
        """Return the ``scan.trigger.arm`` command tree.

        Sub-properties/methods:
            - ``.clear()``: The ``scan.trigger.arm.clear()`` function.
            - ``.set()``: The ``scan.trigger.arm.set()`` function.
            - ``.stimulus``: The ``scan.trigger.arm.stimulus`` attribute.
        """
        return self._arm

    @property
    def channel(self) -> ScanTriggerChannel:
        """Return the ``scan.trigger.channel`` command tree.

        Sub-properties/methods:
            - ``.clear()``: The ``scan.trigger.channel.clear()`` function.
            - ``.set()``: The ``scan.trigger.channel.set()`` function.
            - ``.stimulus``: The ``scan.trigger.channel.stimulus`` attribute.
        """
        return self._channel

    @property
    def measure(self) -> ScanTriggerMeasure:
        """Return the ``scan.trigger.measure`` command tree.

        Sub-properties/methods:
            - ``.clear()``: The ``scan.trigger.measure.clear()`` function.
            - ``.set()``: The ``scan.trigger.measure.set()`` function.
            - ``.stimulus``: The ``scan.trigger.measure.stimulus`` attribute.
        """
        return self._measure

    @property
    def sequence(self) -> ScanTriggerSequence:
        """Return the ``scan.trigger.sequence`` command tree.

        Sub-properties/methods:
            - ``.clear()``: The ``scan.trigger.sequence.clear()`` function.
            - ``.set()``: The ``scan.trigger.sequence.set()`` function.
            - ``.stimulus``: The ``scan.trigger.sequence.stimulus`` attribute.
        """
        return self._sequence

    def clear(self) -> None:
        """Run the ``scan.trigger.clear()`` function.

        **Description:**
            - This function clears the trigger model.

        **TSP Syntax:**

        ::

            - scan.trigger.clear()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Scan(BaseTSPCmd):
    """The ``scan`` command tree.

    Properties/methods:
        - ``.abort()``: The ``scan.abort()`` function.
        - ``.add()``: The ``scan.add()`` function.
        - ``.addimagestep()``: The ``scan.addimagestep()`` function.
        - ``.addwrite()``: The ``scan.addwrite()`` function.
        - ``.background()``: The ``scan.background()`` function.
        - ``.bypass``: The ``scan.bypass`` attribute.
        - ``.create()``: The ``scan.create()`` function.
        - ``.execute()``: The ``scan.execute()`` function.
        - ``.list()``: The ``scan.list()`` function.
        - ``.measurecount``: The ``scan.measurecount`` attribute.
        - ``.mode``: The ``scan.mode`` attribute.
        - ``.nobufferbackground()``: The ``scan.nobufferbackground()`` function.
        - ``.nobufferexecute()``: The ``scan.nobufferexecute()`` function.
        - ``.reset()``: The ``scan.reset()`` function.
        - ``.scancount``: The ``scan.scancount`` attribute.
        - ``.state()``: The ``scan.state()`` function.
        - ``.stepcount``: The ``scan.stepcount`` attribute.
        - ``.trigger``: The ``scan.trigger`` command tree.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "scan") -> None:
        super().__init__(device, cmd_syntax)
        self._trigger = ScanTrigger(device, f"{self._cmd_syntax}.trigger")

    @property
    def bypass(self) -> str:
        """Access the ``scan.bypass`` attribute.

        **Description:**
            - This attribute indicates whether the first channel of the scan waits for the channel
              stimulus event to be satisfied before closing.

        **Usage:**
            - Accessing this property will send the ``print(scan.bypass)`` query.
            - Setting this property to a value will send the ``scan.bypass = value`` command.

        **TSP Syntax:**

        ::

            - scan.bypass = value
            - print(scan.bypass)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".bypass"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.bypass)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.bypass`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @bypass.setter
    def bypass(self, value: Union[str, float]) -> None:
        """Access the ``scan.bypass`` attribute.

        **Description:**
            - This attribute indicates whether the first channel of the scan waits for the channel
              stimulus event to be satisfied before closing.

        **Usage:**
            - Accessing this property will send the ``print(scan.bypass)`` query.
            - Setting this property to a value will send the ``scan.bypass = value`` command.

        **TSP Syntax:**

        ::

            - scan.bypass = value
            - print(scan.bypass)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".bypass", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.bypass = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.bypass`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def measurecount(self) -> str:
        """Access the ``scan.measurecount`` attribute.

        **Description:**
            - This attribute sets the number of iterations performed when a scanning measurement is
              requested.

        **Usage:**
            - Accessing this property will send the ``print(scan.measurecount)`` query.
            - Setting this property to a value will send the ``scan.measurecount = value`` command.

        **TSP Syntax:**

        ::

            - scan.measurecount = value
            - print(scan.measurecount)

        **Info:**
            - ``count``, the count value being used or read; valid range is 1 to 450000.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".measurecount"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.measurecount)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.measurecount`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @measurecount.setter
    def measurecount(self, value: Union[str, float]) -> None:
        """Access the ``scan.measurecount`` attribute.

        **Description:**
            - This attribute sets the number of iterations performed when a scanning measurement is
              requested.

        **Usage:**
            - Accessing this property will send the ``print(scan.measurecount)`` query.
            - Setting this property to a value will send the ``scan.measurecount = value`` command.

        **TSP Syntax:**

        ::

            - scan.measurecount = value
            - print(scan.measurecount)

        **Info:**
            - ``count``, the count value being used or read; valid range is 1 to 450000.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".measurecount", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.measurecount = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.measurecount`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def mode(self) -> str:
        """Access the ``scan.mode`` attribute.

        **Description:**
            - This attribute sets the relay action when the scan starts.

        **Usage:**
            - Accessing this property will send the ``print(scan.mode)`` query.
            - Setting this property to a value will send the ``scan.mode = value`` command.

        **TSP Syntax:**

        ::

            - scan.mode = value
            - print(scan.mode)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".mode"
            return self._device.query(f"print({self._cmd_syntax}.mode)")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @mode.setter
    def mode(self, value: Union[str, float]) -> None:
        """Access the ``scan.mode`` attribute.

        **Description:**
            - This attribute sets the relay action when the scan starts.

        **Usage:**
            - Accessing this property will send the ``print(scan.mode)`` query.
            - Setting this property to a value will send the ``scan.mode = value`` command.

        **TSP Syntax:**

        ::

            - scan.mode = value
            - print(scan.mode)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".mode", value
                )
            else:
                self._device.write(f"{self._cmd_syntax}.mode = {value}")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def scancount(self) -> str:
        """Access the ``scan.scancount`` attribute.

        **Description:**
            - This attribute sets the number of times the scan is repeated.

        **Usage:**
            - Accessing this property will send the ``print(scan.scancount)`` query.
            - Setting this property to a value will send the ``scan.scancount = value`` command.

        **TSP Syntax:**

        ::

            - scan.scancount = value
            - print(scan.scancount)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".scancount"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.scancount)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.scancount`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @scancount.setter
    def scancount(self, value: Union[str, float]) -> None:
        """Access the ``scan.scancount`` attribute.

        **Description:**
            - This attribute sets the number of times the scan is repeated.

        **Usage:**
            - Accessing this property will send the ``print(scan.scancount)`` query.
            - Setting this property to a value will send the ``scan.scancount = value`` command.

        **TSP Syntax:**

        ::

            - scan.scancount = value
            - print(scan.scancount)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".scancount", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.scancount = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.scancount`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def stepcount(self) -> str:
        """Access the ``scan.stepcount`` attribute.

        **Description:**
            - This attribute returns the number of steps in the present scan.

        **Usage:**
            - Accessing this property will send the ``print(scan.stepcount)`` query.

        **TSP Syntax:**

        ::

            - print(scan.stepcount)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stepcount"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stepcount)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stepcount`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def trigger(self) -> ScanTrigger:
        """Return the ``scan.trigger`` command tree.

        Sub-properties/methods:
            - ``.arm``: The ``scan.trigger.arm`` command tree.
            - ``.channel``: The ``scan.trigger.channel`` command tree.
            - ``.clear()``: The ``scan.trigger.clear()`` function.
            - ``.measure``: The ``scan.trigger.measure`` command tree.
            - ``.sequence``: The ``scan.trigger.sequence`` command tree.
        """
        return self._trigger

    def abort(self) -> None:
        """Run the ``scan.abort()`` function.

        **Description:**
            - This function aborts a running background scan.

        **TSP Syntax:**

        ::

            - scan.abort()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.abort()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.abort()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def add(self, channel_list: str, width: Optional[str] = None) -> None:
        """Run the ``scan.add()`` function.

        **Description:**
            - This function adds channels to the scan list.

        **TSP Syntax:**

        ::

            - scan.add()

        Args:
            channel_list: String specifying channels to add using normal channel list syntax.
            width (optional): Value that specifies the width of the channel read to use with items
                in channelList.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"',
                    width,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.add({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.add()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def addimagestep(self, channel_list: str, dmm_config: Optional[str] = None) -> None:
        """Run the ``scan.addimagestep()`` function.

        **Description:**
            - This function allows you to include multiple channels in a single scan step.

        **TSP Syntax:**

        ::

            - scan.addimagestep()

        Args:
            channel_list: String specifying a list of channels.
            dmm_config (optional): String specifying a DMM configuration.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"',
                    f'"{dmm_config}"' if dmm_config is not None else None,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.addimagestep({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.addimagestep()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def addwrite(self, channel_list: str, write_value: str, width: Optional[str] = None) -> None:
        """Run the ``scan.addwrite()`` function.

        **Description:**
            - This function writes a specified value to a channel at the added step in the scan.

        **TSP Syntax:**

        ::

            - scan.addwrite()

        Args:
            channel_list: String specifying channels to add using normal channel list syntax.
            write_value: The value to write to the channel for this scan step.
            width (optional): Specifies the width of the channel write to use with items in
                channelList.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"',
                    write_value,
                    width,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.addwrite({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.addwrite()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def background(self, buffer_var: Optional[str] = None) -> str:
        """Run the ``scan.background()`` function.

        **Description:**
            - This function starts a scan and runs the scan in the background.

        **TSP Syntax:**

        ::

            - scan.background()

        Args:
            buffer_var (optional): The reading buffer used during scanning to store the readings; if
                a buffer is not specified, no readings are stored during the scan.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (buffer_var,) if x is not None)
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.background({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.background()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def create(self, channel_list: str, dmm_config: Optional[str] = None) -> None:
        """Run the ``scan.create()`` function.

        **Description:**
            - This function deletes the existing scan list and creates a new list of channels and
              channel patterns to scan.

        **TSP Syntax:**

        ::

            - scan.create()

        Args:
            channel_list: String specifying channels to add.
            dmm_config (optional): The DMM configuration to use with items in the channelList.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    channel_list,
                    dmm_config,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.create({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.create()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def execute(self, buffer_var: Optional[str] = None) -> str:
        """Run the ``scan.execute()`` function.

        **Description:**
            - This function starts the scan immediately in the foreground with a configured scan
              list.

        **TSP Syntax:**

        ::

            - scan.execute()

        Args:
            buffer_var (optional): A reading buffer used during scanning to store the readings; if a
                buffer is not specified, no readings are stored during the scan.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (buffer_var,) if x is not None)
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.execute({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.execute()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def list(self) -> str:
        """Run the ``scan.list()`` function.

        **Description:**
            - This function queries the active scan list.

        **TSP Syntax:**

        ::

            - scan.list()

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.list())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.list()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def nobufferbackground(self) -> str:
        """Run the ``scan.nobufferbackground()`` function.

        **Description:**
            - This function starts a scan in background mode and specifies that no reading buffer is
              used during scanning.

        **TSP Syntax:**

        ::

            - scan.nobufferbackground()

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.nobufferbackground())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.nobufferbackground()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def nobufferexecute(self) -> None:
        """Run the ``scan.nobufferexecute()`` function.

        **Description:**
            - This function starts a scan immediately and specifies that no reading buffer is used
              during scanning.

        **TSP Syntax:**

        ::

            - scan.nobufferexecute()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.nobufferexecute()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.nobufferexecute()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def reset(self) -> None:
        """Run the ``scan.reset()`` function.

        **Description:**
            - This function resets the trigger model and scan list settings to their factory default
              settings.

        **TSP Syntax:**

        ::

            - scan.reset()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.reset()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.reset()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def state(self) -> str:
        """Run the ``scan.state()`` function.

        **Description:**
            - This function provides the present state of a running background scan.

        **TSP Syntax:**

        ::

            - scan.state()

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.state())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.state()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The clear commands module.

These commands are used in the following models:
LPD6, MDO3, MSO2, MSO4, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - CLEAR
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Clear(SCPICmdWriteNoArguments):
    """The ``CLEAR`` command.

    **Description:**
        - This command clears acquisitions, measurements, and waveforms.

    **Usage:**
        - Using the ``.write()`` method will send the ``CLEAR`` command.

    **SCPI Syntax:**

    ::

        - CLEAR
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "CLEAR") -> None:
        super().__init__(device, cmd_syntax)



"""The wfmoutpre commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - WFMOutpre:BIT_Nr <NR1>
    - WFMOutpre:BIT_Nr?
    - WFMOutpre:BN_Fmt {RI|RP|FP}
    - WFMOutpre:BN_Fmt?
    - WFMOutpre:BYT_Nr <NR1>
    - WFMOutpre:BYT_Nr?
    - WFMOutpre:BYT_Or {LSB|MSB}
    - WFMOutpre:BYT_Or?
    - WFMOutpre:ENCdg {ASCii|BINary}
    - WFMOutpre:ENCdg?
    - WFMOutpre:NR_FR?
    - WFMOutpre:NR_Pt?
    - WFMOutpre:PT_Fmt?
    - WFMOutpre:PT_ORder?
    - WFMOutpre:PT_Off?
    - WFMOutpre:WFId?
    - WFMOutpre:XINcr?
    - WFMOutpre:XUNit?
    - WFMOutpre:XZEro?
    - WFMOutpre:YMUlt?
    - WFMOutpre:YOFf?
    - WFMOutpre:YUNit?
    - WFMOutpre:YZEro?
    - WFMOutpre?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class WfmoutpreYzero(SCPICmdRead):
    """The ``WFMOutpre:YZEro`` command.

    **Description:**
        - This query-only command returns the combined vertical position and offset for the source
          waveform specified by ``DATA:SOURCE``. This represents a departure from previous
          instruments where the ``:YZEro`` value represented the vertical position in vertical units
          and the ``:YOFf`` value represented the vertical offset in digitizing levels. For this
          instrument family, the value of ``:YOFf`` is always 0.0. An error is reported if the
          ``DATA:SOURCE`` waveform does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:YZEro?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YZEro?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:YZEro?
    """


class WfmoutpreYunit(SCPICmdRead):
    """The ``WFMOutpre:YUNit`` command.

    **Description:**
        - This query-only command returns the vertical units for the waveform specified by the
          ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
          exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:YUNit?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YUNit?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:YUNit?
    """


class WfmoutpreYoff(SCPICmdRead):
    """The ``WFMOutpre:YOFf`` command.

    **Description:**
        - This query-only command returns the vertical offset of the source specified by
          ``DATA:SOURCE``. For this instrument family, the value returned is always 0.0 as the
          offset is combined with the ``:YZEro`` value. An error is reported if the ``DATA:SOURCE``
          waveform does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:YOFf?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YOFf?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:YOFf?
    """


class WfmoutpreYmult(SCPICmdRead):
    """The ``WFMOutpre:YMUlt`` command.

    **Description:**
        - This query-only command returns the vertical scale factor per digitizing level in units
          specified by ``WFMOutpre:YUNit`` for the waveform specified by the ``DATA:SOURCE``
          command. For those formats in which ``WFMOUTPRE:BYT_NR`` is important (all non-floating
          point formats), ``WFMOutpre:YMUlt?`` must take the location of the binary point implied by
          ``BYT_NR`` into consideration. An error is reported if the ``DATa:SOUrce`` waveform does
          not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:YMUlt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YMUlt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:YMUlt?
    """


class WfmoutpreXzero(SCPICmdRead):
    """The ``WFMOutpre:XZEro`` command.

    **Description:**
        - This query-only command returns the sub-sample time between the trigger sample (designated
          by ``PT_OFF``) and the occurrence of the actual trigger for the waveform specified by the
          ``DATA:SOURCE`` command. This value is in units of ``WFMOutpre:XUNit``. An error is
          reported if the ``DATa:SOUrce`` waveform does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:XZEro?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:XZEro?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:XZEro?
    """


class WfmoutpreXunit(SCPICmdRead):
    """The ``WFMOutpre:XUNit`` command.

    **Description:**
        - This query-only command returns the horizontal units for the waveform specified by the
          ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
          exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:XUNit?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:XUNit?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:XUNit?
    """


class WfmoutpreXincr(SCPICmdRead):
    """The ``WFMOutpre:XINcr`` command.

    **Description:**
        - This query-only command returns the horizontal point spacing in units of
          ``WFMOutpre:XUNit`` for the waveform specified by the ``DATA:SOURCE`` command. This value
          corresponds to the sampling interval. An error is reported if the ``DATa:SOUrce`` waveform
          does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:XINcr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:XINcr?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:XINcr?
    """


class WfmoutpreWfid(SCPICmdRead):
    """The ``WFMOutpre:WFId`` command.

    **Description:**
        - This query-only command returns a string describing several aspects of the acquisition
          parameters for the waveform specified by the ``DATA:SOURCE`` command. An error is reported
          if the ``DATa:SOUrce`` waveform does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:WFId?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:WFId?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:WFId?
    """


class WfmoutprePtOff(SCPICmdRead):
    """The ``WFMOutpre:PT_Off`` command.

    **Description:**
        - This query-only command returns the trigger point relative to ``DATA:START`` for the
          waveform specified by the ``DATA:SOURCE`` command.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:PT_Off?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Off?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:PT_Off?
    """


class WfmoutprePtOrder(SCPICmdRead):
    """The ``WFMOutpre:PT_ORder`` command.

    **Description:**
        - This query-only command specifies whether the source waveform is Fast Acquisition. A Fast
          Acquisition waveform is stored as a 500 (vertical) by 1000 (horizontal) point bitmap. Each
          point represents display intensity for that screen location. Only CURVe? query functions
          are allowed on Fast Acquisition waveforms. When the ``WFMOutpre:PT_ORder?`` query returns
          Row, this indicates that the source is a Fast Acquisition waveform (and that each of 1000
          possible horizontal columns being transmitted contains 500 vertical points. Note: 500
          might vary based on how many channels enabled from same FastAcq group.) and waveform
          points are transmitted in the following order: left to right; then top to bottom. When the
          ``WFMOutpre:PT_ORder?`` query returns Linear, this indicates that the source is not a Fast
          Acquisition waveform (and that each horizontal column being sent contains only one
          vertical point). Note that waveform points are transmitted in the following order: top to
          bottom, then left to right.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:PT_ORder?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_ORder?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:PT_ORder?
    """


class WfmoutprePtFmt(SCPICmdRead):
    """The ``WFMOutpre:PT_Fmt`` command.

    **Description:**
        - This query-only command returns the point format for the waveform specified by the
          ``DATA:SOURCE`` command. The format specifies a set of equations describing how the scale
          factors in the preamble are used to give meaning to the CURVE data points. An error is
          reported if the ``DATa:SOUrce`` waveform does not exist.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:PT_Fmt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Fmt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:PT_Fmt?
    """


class WfmoutpreNrPt(SCPICmdRead):
    """The ``WFMOutpre:NR_Pt`` command.

    **Description:**
        - This query-only command returns the number of points for the ``DATA:SOURCE`` waveform that
          will be transmitted in response to a CURVE? query.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:NR_Pt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:NR_Pt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:NR_Pt?
    """


class WfmoutpreNrFr(SCPICmdRead):
    """The ``WFMOutpre:NR_FR`` command.

    **Description:**
        - This query-only command returns the number of frames for the ``DATA:SOURCE`` waveform
          transmitted in response to a CURVE? query.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:NR_FR?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:NR_FR?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre:NR_FR?
    """


class WfmoutpreEncdg(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:ENCdg`` command.

    **Description:**
        - This command sets or queries the type of encoding for outgoing waveforms.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:ENCdg?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:ENCdg?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:ENCdg value`` command.

    **SCPI Syntax:**

    ::

        - WFMOutpre:ENCdg {ASCii|BINary}
        - WFMOutpre:ENCdg?

    **Info:**
        - ``ASCii`` specifies that the outgoing data is to be in ASCII format. Waveforms internally
          stored as integers will be sent as <NR1> numbers, while those stored as floating point
          will be sent as <NR3> numbers.
        - ``BINary`` specifies that outgoing data is to be in a binary format whose further
          specification is determined by ``WFMOUTPRE:BYT_NR``, ``WFMOUTPRE:BIT_NR``,
          ``WFMOUTPRE:BN_FMT`` and ``WFMOUTPRE:BYT_OR``.
    """


class WfmoutpreBytOr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BYT_Or`` command.

    **Description:**
        - This command sets or queries which byte of binary waveform data is transmitted first,
          during a waveform data transfer, when data points require more than one byte. This
          specification only has meaning when ``WFMOUTPRE:ENCDG`` is set to BIN.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Or?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Or?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Or value`` command.

    **SCPI Syntax:**

    ::

        - WFMOutpre:BYT_Or {LSB|MSB}
        - WFMOutpre:BYT_Or?

    **Info:**
        - ``LSB`` specifies that the least significant byte will be transmitted first.
        - ``MSB`` specifies that the most significant byte will be transmitted first.
    """


class WfmoutpreBytNr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BYT_Nr`` command.

    **Description:**
        - This command sets or queries the binary field data width (bytes per point) for the
          waveform specified by the ``DATA:SOURCE`` command. Note that values will be constrained
          according to the underlying waveform data. This specification is only meaningful when
          ``WFMOUTPRE:ENCDG`` is set to BIN, and ``WFMOUTPRE:BN_FMT`` is set to either RI or RP.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Nr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Nr?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Nr value`` command.

    **SCPI Syntax:**

    ::

        - WFMOutpre:BYT_Nr <NR1>
        - WFMOutpre:BYT_Nr?

    **Info:**
        - ``<NR1>`` is the number of bytes per data point and can be 1, 2 or 8. A value of 1 or 2
          bytes per waveform point indicates channel data; 8 bytes per waveform point indicate pixel
          map (fast acquisition) data.
    """


class WfmoutpreBnFmt(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BN_Fmt`` command.

    **Description:**
        - This command sets or queries the format of binary data for outgoing waveforms specified by
          the ``DATA:SOURCE`` command.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:BN_Fmt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BN_Fmt?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BN_Fmt value`` command.

    **SCPI Syntax:**

    ::

        - WFMOutpre:BN_Fmt {RI|RP|FP}
        - WFMOutpre:BN_Fmt?

    **Info:**
        - ``RI`` specifies signed integer data point representation.
        - ``RP`` specifies positive integer data point representation.
        - ``FP`` specifies floating point representation.
    """


class WfmoutpreBitNr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BIT_Nr`` command.

    **Description:**
        - This command sets and queries the number of bits per waveform point that outgoing
          waveforms contain, as specified by the ``DATA:SOURCE`` command. Note that values will be
          constrained according to the underlying waveform data. This specification is only
          meaningful when ``WFMOUTPRE:ENCDG`` is set to BIN and ``WFMOUTPRE:BN_FMT`` is set to
          either RI or RP.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre:BIT_Nr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BIT_Nr?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BIT_Nr value`` command.

    **SCPI Syntax:**

    ::

        - WFMOutpre:BIT_Nr <NR1>
        - WFMOutpre:BIT_Nr?

    **Info:**
        - ``<NR1>`` number of bits per data point can be 8 or 16.
    """


#  pylint: disable=too-many-instance-attributes
class Wfmoutpre(SCPICmdRead):
    """The ``WFMOutpre`` command.

    **Description:**
        - This query-only command queries the waveform formatting data for the waveform specified by
          the ``DATA:SOURCE`` command. The preamble components are considered to be of two types;
          formatting and interpretation. The formatting components are: ENCdg, ``BN_Fmt``,
          ``BYT_Or``, ``BYT_Nr``, ``BIT_Nr``. The interpretation components are derived from the
          ``DATa:SOUrce`` specified waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``WFMOutpre?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - WFMOutpre?

    Properties:
        - ``.bit_nr``: The ``WFMOutpre:BIT_Nr`` command.
        - ``.bn_fmt``: The ``WFMOutpre:BN_Fmt`` command.
        - ``.byt_nr``: The ``WFMOutpre:BYT_Nr`` command.
        - ``.byt_or``: The ``WFMOutpre:BYT_Or`` command.
        - ``.encdg``: The ``WFMOutpre:ENCdg`` command.
        - ``.nr_fr``: The ``WFMOutpre:NR_FR`` command.
        - ``.nr_pt``: The ``WFMOutpre:NR_Pt`` command.
        - ``.pt_fmt``: The ``WFMOutpre:PT_Fmt`` command.
        - ``.pt_order``: The ``WFMOutpre:PT_ORder`` command.
        - ``.pt_off``: The ``WFMOutpre:PT_Off`` command.
        - ``.wfid``: The ``WFMOutpre:WFId`` command.
        - ``.xincr``: The ``WFMOutpre:XINcr`` command.
        - ``.xunit``: The ``WFMOutpre:XUNit`` command.
        - ``.xzero``: The ``WFMOutpre:XZEro`` command.
        - ``.ymult``: The ``WFMOutpre:YMUlt`` command.
        - ``.yoff``: The ``WFMOutpre:YOFf`` command.
        - ``.yunit``: The ``WFMOutpre:YUNit`` command.
        - ``.yzero``: The ``WFMOutpre:YZEro`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "WFMOutpre") -> None:
        super().__init__(device, cmd_syntax)
        self._bit_nr = WfmoutpreBitNr(device, f"{self._cmd_syntax}:BIT_Nr")
        self._bn_fmt = WfmoutpreBnFmt(device, f"{self._cmd_syntax}:BN_Fmt")
        self._byt_nr = WfmoutpreBytNr(device, f"{self._cmd_syntax}:BYT_Nr")
        self._byt_or = WfmoutpreBytOr(device, f"{self._cmd_syntax}:BYT_Or")
        self._encdg = WfmoutpreEncdg(device, f"{self._cmd_syntax}:ENCdg")
        self._nr_fr = WfmoutpreNrFr(device, f"{self._cmd_syntax}:NR_FR")
        self._nr_pt = WfmoutpreNrPt(device, f"{self._cmd_syntax}:NR_Pt")
        self._pt_fmt = WfmoutprePtFmt(device, f"{self._cmd_syntax}:PT_Fmt")
        self._pt_order = WfmoutprePtOrder(device, f"{self._cmd_syntax}:PT_ORder")
        self._pt_off = WfmoutprePtOff(device, f"{self._cmd_syntax}:PT_Off")
        self._wfid = WfmoutpreWfid(device, f"{self._cmd_syntax}:WFId")
        self._xincr = WfmoutpreXincr(device, f"{self._cmd_syntax}:XINcr")
        self._xunit = WfmoutpreXunit(device, f"{self._cmd_syntax}:XUNit")
        self._xzero = WfmoutpreXzero(device, f"{self._cmd_syntax}:XZEro")
        self._ymult = WfmoutpreYmult(device, f"{self._cmd_syntax}:YMUlt")
        self._yoff = WfmoutpreYoff(device, f"{self._cmd_syntax}:YOFf")
        self._yunit = WfmoutpreYunit(device, f"{self._cmd_syntax}:YUNit")
        self._yzero = WfmoutpreYzero(device, f"{self._cmd_syntax}:YZEro")

    @property
    def bit_nr(self) -> WfmoutpreBitNr:
        """Return the ``WFMOutpre:BIT_Nr`` command.

        **Description:**
            - This command sets and queries the number of bits per waveform point that outgoing
              waveforms contain, as specified by the ``DATA:SOURCE`` command. Note that values will
              be constrained according to the underlying waveform data. This specification is only
              meaningful when ``WFMOUTPRE:ENCDG`` is set to BIN and ``WFMOUTPRE:BN_FMT`` is set to
              either RI or RP.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:BIT_Nr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BIT_Nr?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BIT_Nr value`` command.

        **SCPI Syntax:**

        ::

            - WFMOutpre:BIT_Nr <NR1>
            - WFMOutpre:BIT_Nr?

        **Info:**
            - ``<NR1>`` number of bits per data point can be 8 or 16.
        """
        return self._bit_nr

    @property
    def bn_fmt(self) -> WfmoutpreBnFmt:
        """Return the ``WFMOutpre:BN_Fmt`` command.

        **Description:**
            - This command sets or queries the format of binary data for outgoing waveforms
              specified by the ``DATA:SOURCE`` command.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:BN_Fmt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BN_Fmt?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BN_Fmt value`` command.

        **SCPI Syntax:**

        ::

            - WFMOutpre:BN_Fmt {RI|RP|FP}
            - WFMOutpre:BN_Fmt?

        **Info:**
            - ``RI`` specifies signed integer data point representation.
            - ``RP`` specifies positive integer data point representation.
            - ``FP`` specifies floating point representation.
        """
        return self._bn_fmt

    @property
    def byt_nr(self) -> WfmoutpreBytNr:
        """Return the ``WFMOutpre:BYT_Nr`` command.

        **Description:**
            - This command sets or queries the binary field data width (bytes per point) for the
              waveform specified by the ``DATA:SOURCE`` command. Note that values will be
              constrained according to the underlying waveform data. This specification is only
              meaningful when ``WFMOUTPRE:ENCDG`` is set to BIN, and ``WFMOUTPRE:BN_FMT`` is set to
              either RI or RP.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Nr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Nr?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Nr value`` command.

        **SCPI Syntax:**

        ::

            - WFMOutpre:BYT_Nr <NR1>
            - WFMOutpre:BYT_Nr?

        **Info:**
            - ``<NR1>`` is the number of bytes per data point and can be 1, 2 or 8. A value of 1 or
              2 bytes per waveform point indicates channel data; 8 bytes per waveform point indicate
              pixel map (fast acquisition) data.
        """
        return self._byt_nr

    @property
    def byt_or(self) -> WfmoutpreBytOr:
        """Return the ``WFMOutpre:BYT_Or`` command.

        **Description:**
            - This command sets or queries which byte of binary waveform data is transmitted first,
              during a waveform data transfer, when data points require more than one byte. This
              specification only has meaning when ``WFMOUTPRE:ENCDG`` is set to BIN.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Or?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Or?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Or value`` command.

        **SCPI Syntax:**

        ::

            - WFMOutpre:BYT_Or {LSB|MSB}
            - WFMOutpre:BYT_Or?

        **Info:**
            - ``LSB`` specifies that the least significant byte will be transmitted first.
            - ``MSB`` specifies that the most significant byte will be transmitted first.
        """
        return self._byt_or

    @property
    def encdg(self) -> WfmoutpreEncdg:
        """Return the ``WFMOutpre:ENCdg`` command.

        **Description:**
            - This command sets or queries the type of encoding for outgoing waveforms.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:ENCdg?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:ENCdg?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:ENCdg value`` command.

        **SCPI Syntax:**

        ::

            - WFMOutpre:ENCdg {ASCii|BINary}
            - WFMOutpre:ENCdg?

        **Info:**
            - ``ASCii`` specifies that the outgoing data is to be in ASCII format. Waveforms
              internally stored as integers will be sent as <NR1> numbers, while those stored as
              floating point will be sent as <NR3> numbers.
            - ``BINary`` specifies that outgoing data is to be in a binary format whose further
              specification is determined by ``WFMOUTPRE:BYT_NR``, ``WFMOUTPRE:BIT_NR``,
              ``WFMOUTPRE:BN_FMT`` and ``WFMOUTPRE:BYT_OR``.
        """
        return self._encdg

    @property
    def nr_fr(self) -> WfmoutpreNrFr:
        """Return the ``WFMOutpre:NR_FR`` command.

        **Description:**
            - This query-only command returns the number of frames for the ``DATA:SOURCE`` waveform
              transmitted in response to a CURVE? query.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:NR_FR?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:NR_FR?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:NR_FR?
        """
        return self._nr_fr

    @property
    def nr_pt(self) -> WfmoutpreNrPt:
        """Return the ``WFMOutpre:NR_Pt`` command.

        **Description:**
            - This query-only command returns the number of points for the ``DATA:SOURCE`` waveform
              that will be transmitted in response to a CURVE? query.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:NR_Pt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:NR_Pt?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:NR_Pt?
        """
        return self._nr_pt

    @property
    def pt_fmt(self) -> WfmoutprePtFmt:
        """Return the ``WFMOutpre:PT_Fmt`` command.

        **Description:**
            - This query-only command returns the point format for the waveform specified by the
              ``DATA:SOURCE`` command. The format specifies a set of equations describing how the
              scale factors in the preamble are used to give meaning to the CURVE data points. An
              error is reported if the ``DATa:SOUrce`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:PT_Fmt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Fmt?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:PT_Fmt?
        """
        return self._pt_fmt

    @property
    def pt_order(self) -> WfmoutprePtOrder:
        """Return the ``WFMOutpre:PT_ORder`` command.

        **Description:**
            - This query-only command specifies whether the source waveform is Fast Acquisition. A
              Fast Acquisition waveform is stored as a 500 (vertical) by 1000 (horizontal) point
              bitmap. Each point represents display intensity for that screen location. Only CURVe?
              query functions are allowed on Fast Acquisition waveforms. When the
              ``WFMOutpre:PT_ORder?`` query returns Row, this indicates that the source is a Fast
              Acquisition waveform (and that each of 1000 possible horizontal columns being
              transmitted contains 500 vertical points. Note: 500 might vary based on how many
              channels enabled from same FastAcq group.) and waveform points are transmitted in the
              following order: left to right; then top to bottom. When the ``WFMOutpre:PT_ORder?``
              query returns Linear, this indicates that the source is not a Fast Acquisition
              waveform (and that each horizontal column being sent contains only one vertical
              point). Note that waveform points are transmitted in the following order: top to
              bottom, then left to right.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:PT_ORder?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_ORder?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:PT_ORder?
        """
        return self._pt_order

    @property
    def pt_off(self) -> WfmoutprePtOff:
        """Return the ``WFMOutpre:PT_Off`` command.

        **Description:**
            - This query-only command returns the trigger point relative to ``DATA:START`` for the
              waveform specified by the ``DATA:SOURCE`` command.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:PT_Off?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Off?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:PT_Off?
        """
        return self._pt_off

    @property
    def wfid(self) -> WfmoutpreWfid:
        """Return the ``WFMOutpre:WFId`` command.

        **Description:**
            - This query-only command returns a string describing several aspects of the acquisition
              parameters for the waveform specified by the ``DATA:SOURCE`` command. An error is
              reported if the ``DATa:SOUrce`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:WFId?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:WFId?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:WFId?
        """
        return self._wfid

    @property
    def xincr(self) -> WfmoutpreXincr:
        """Return the ``WFMOutpre:XINcr`` command.

        **Description:**
            - This query-only command returns the horizontal point spacing in units of
              ``WFMOutpre:XUNit`` for the waveform specified by the ``DATA:SOURCE`` command. This
              value corresponds to the sampling interval. An error is reported if the
              ``DATa:SOUrce`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:XINcr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:XINcr?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:XINcr?
        """
        return self._xincr

    @property
    def xunit(self) -> WfmoutpreXunit:
        """Return the ``WFMOutpre:XUNit`` command.

        **Description:**
            - This query-only command returns the horizontal units for the waveform specified by the
              ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
              exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:XUNit?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:XUNit?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:XUNit?
        """
        return self._xunit

    @property
    def xzero(self) -> WfmoutpreXzero:
        """Return the ``WFMOutpre:XZEro`` command.

        **Description:**
            - This query-only command returns the sub-sample time between the trigger sample
              (designated by ``PT_OFF``) and the occurrence of the actual trigger for the waveform
              specified by the ``DATA:SOURCE`` command. This value is in units of
              ``WFMOutpre:XUNit``. An error is reported if the ``DATa:SOUrce`` waveform does not
              exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:XZEro?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:XZEro?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:XZEro?
        """
        return self._xzero

    @property
    def ymult(self) -> WfmoutpreYmult:
        """Return the ``WFMOutpre:YMUlt`` command.

        **Description:**
            - This query-only command returns the vertical scale factor per digitizing level in
              units specified by ``WFMOutpre:YUNit`` for the waveform specified by the
              ``DATA:SOURCE`` command. For those formats in which ``WFMOUTPRE:BYT_NR`` is important
              (all non-floating point formats), ``WFMOutpre:YMUlt?`` must take the location of the
              binary point implied by ``BYT_NR`` into consideration. An error is reported if the
              ``DATa:SOUrce`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:YMUlt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YMUlt?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:YMUlt?
        """
        return self._ymult

    @property
    def yoff(self) -> WfmoutpreYoff:
        """Return the ``WFMOutpre:YOFf`` command.

        **Description:**
            - This query-only command returns the vertical offset of the source specified by
              ``DATA:SOURCE``. For this instrument family, the value returned is always 0.0 as the
              offset is combined with the ``:YZEro`` value. An error is reported if the
              ``DATA:SOURCE`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:YOFf?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YOFf?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:YOFf?
        """
        return self._yoff

    @property
    def yunit(self) -> WfmoutpreYunit:
        """Return the ``WFMOutpre:YUNit`` command.

        **Description:**
            - This query-only command returns the vertical units for the waveform specified by the
              ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
              exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:YUNit?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YUNit?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:YUNit?
        """
        return self._yunit

    @property
    def yzero(self) -> WfmoutpreYzero:
        """Return the ``WFMOutpre:YZEro`` command.

        **Description:**
            - This query-only command returns the combined vertical position and offset for the
              source waveform specified by ``DATA:SOURCE``. This represents a departure from
              previous instruments where the ``:YZEro`` value represented the vertical position in
              vertical units and the ``:YOFf`` value represented the vertical offset in digitizing
              levels. For this instrument family, the value of ``:YOFf`` is always 0.0. An error is
              reported if the ``DATA:SOURCE`` waveform does not exist.

        **Usage:**
            - Using the ``.query()`` method will send the ``WFMOutpre:YZEro?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YZEro?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - WFMOutpre:YZEro?
        """
        return self._yzero
